@@ -1,11 +1,14 @@
 import json
+import os
 import sys
 import textwrap
 from collections import namedtuple
+from itertools import chain
 
 import pkg_resources
 import py
 import six
+from filelock import FileLock, Timeout
 
 import tox
 from tox.config import DepConfig, get_py_project_toml
@@ -17,7 +20,7 @@ BuildInfo = namedtuple("BuildInfo", ["requires", "backend_module", "backend_obje
 def tox_package(session, venv):
     """Build an sdist at first call return that for all calls"""
     if not hasattr(session, "package"):
-        session.package = get_package(session)
+        session.package, session.dist = get_package(session)
     return session.package
 
 
@@ -27,12 +30,68 @@ def get_package(session):
     if config.skipsdist:
         report.info("skipping sdist step")
         return None
+    lock_file = str(
+        session.config.toxworkdir.join("{}.lock".format(session.config.isolated_build_env))
+    )
+    lock = FileLock(lock_file)
+    try:
+        try:
+            lock.acquire(0.0001)
+        except Timeout:
+            report.verbosity0("lock file {} present, will block until released".format(lock_file))
+            lock.acquire()
+        package = acquire_package(config, report, session)
+        session_package = create_session_view(package, config.temp_dir, report)
+        return session_package, package
+    finally:
+        lock.release(force=True)
+
+
+def create_session_view(package, temp_dir, report):
+    """once we build a package we cannot return that directly, as a subsequent call
+    might delete that package (in order to do its own build); therefore we need to
+    return a view of the file that it's not prone to deletion and can be removed when the
+    session ends
+    """
+    if not package:
+        return package
+    package_dir = temp_dir.join("package")
+    package_dir.ensure(dir=True)
+
+    # we'll number the active instances, and use the max value as session folder for a new build
+    # note we cannot change package names as PEP-491 (wheel binary format)
+    # is strict about file name structure
+    exists = [i.basename for i in package_dir.listdir()]
+    file_id = max(chain((0,), (int(i) for i in exists if six.text_type(i).isnumeric())))
+
+    session_dir = package_dir.join(str(file_id + 1))
+    session_dir.ensure(dir=True)
+    session_package = session_dir.join(package.basename)
+
+    # if we can do hard links do that, otherwise just copy
+    links = False
+    if hasattr(os, "link"):
+        try:
+            os.link(str(package), str(session_package))
+            links = True
+        except (OSError, NotImplementedError):
+            pass
+    if not links:
+        package.copy(session_package)
+    operation = "links" if links else "copied"
+    common = session_package.common(package)
+    report.verbosity1(
+        "package {} {} to {} ({})".format(
+            common.bestrelpath(session_package), operation, common.bestrelpath(package), common
+        )
+    )
+    return session_package
+
+
+def acquire_package(config, report, session):
+    """acquire a source distribution (either by loading a local file or triggering a build)"""
     if not config.option.sdistonly and (config.sdistsrc or config.option.installpkg):
-        path = config.option.installpkg
-        if not path:
-            path = config.sdistsrc
-        path = session._resolve_package(path)
-        report.info("using package {!r}, skipping 'sdist' activity ".format(str(path)))
+        path = get_local_package(config, report, session)
     else:
         try:
             path = build_package(config, report, session)
@@ -51,6 +110,15 @@ def get_package(session):
     return path
 
 
+def get_local_package(config, report, session):
+    path = config.option.installpkg
+    if not path:
+        path = config.sdistsrc
+    py_path = py.path.local(session._resolve_package(path))
+    report.info("using package {!r}, skipping 'sdist' activity ".format(str(py_path)))
+    return py_path
+
+
 def build_package(config, report, session):
     if not config.isolated_build:
         return make_sdist_legacy(report, config, session)
@@ -75,10 +143,12 @@ def make_sdist_legacy(report, config, session):
     with session.newaction(None, "packaging") as action:
         action.setactivity("sdist-make", setup)
         session.make_emptydir(config.distdir)
-        action.popen(
+        build_log = action.popen(
             [sys.executable, setup, "sdist", "--formats=zip", "--dist-dir", config.distdir],
             cwd=config.setupdir,
+            returnout=True,
         )
+        report.verbosity2(build_log)
         try:
             return config.distdir.listdir()[0]
         except py.error.ENOENT:
@@ -127,7 +197,7 @@ def build_isolated(config, report, session):
         ) as action:
             package_venv.run_install_command(packages=build_requires_dep, action=action)
         session.finishvenv(package_venv)
-    return perform_isolated_build(build_info, package_venv, session, config)
+    return perform_isolated_build(build_info, package_venv, session, config, report)
 
 
 def get_build_info(folder, report):
@@ -170,7 +240,7 @@ def get_build_info(folder, report):
     return BuildInfo(requires, module, "{}{}".format(module, obj))
 
 
-def perform_isolated_build(build_info, package_venv, session, config):
+def perform_isolated_build(build_info, package_venv, session, config, report):
     with session.newaction(
         package_venv, "perform-isolated-build", package_venv.envconfig.envdir
     ) as action:
@@ -195,6 +265,7 @@ def perform_isolated_build(build_info, package_venv, session, config):
             action=action,
             cwd=session.config.setupdir,
         )
+        report.verbosity2(result)
         return config.distdir.join(result.split("\n")[-2])
 
 
